﻿using System;
using System.Collections.Generic;
using AreaCalculationDomain;

namespace AreaCalculation
{
    public static class CylinderCalculator
    {
        private static CylinderMeasurement _currentCyl;

         public static void UpdateCylinders(IEnumerable<CylinderMeasurement> cyls)
        {
            //do all processing
            foreach (var cyl in cyls)
            {
                _currentCyl = cyl;
                cyl.Psi = GetPsi();
            }
        }

        private static double GetAreaForCylinder()
        {
            switch (_currentCyl.CylinderType)
            {
                case CylinderType.FourFourEightCylinder:
                    return 3.14159*((_currentCyl.WidthA + _currentCyl.WidthB)/2)/2*
                           ((_currentCyl.WidthA + _currentCyl.WidthB)/2/2);
                case CylinderType.SixSixTwelveCylinder:
                    return 3.14159*((_currentCyl.WidthA + _currentCyl.WidthB)/2)/2*
                           ((_currentCyl.WidthA + _currentCyl.WidthB)/2/2);
                case CylinderType.ThreeThreeSixCylinder:
                    return _currentCyl.WidthA*_currentCyl.WidthB;
                case CylinderType.TwoTwoTwoCylinder:
                    return ((_currentCyl.WidthA + _currentCyl.WidthB)/2)*((_currentCyl.WidthA + _currentCyl.WidthB)/2);
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

        private static double GetPsi()
        {
            var area = GetAreaForCylinder();
            return PsiCalculator(area);
        }


        private static int PsiCalculator(double area)
        {
            if (_currentCyl.LoadPounds > 0 && area > 0)
            {
                return (int) (Math.Round(_currentCyl.LoadPounds/area/1000*_currentCyl.ToleranceFactor, 2)*1000);
            }
            return 0;
        }
    }
}