﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using AreaCalculation;
using AreaCalculationDomain;

namespace ConsoleApplication
{
    internal class Program
    {
      private static int _iterations;
        private static void Main(string[] args) {

          _iterations = 50000;
            for (int i = 0; i < 10; i++)
            {
                CalculatePsiWithCSharp();
                CalculatePsiWithFSharp();
                Console.WriteLine();

            } Console.ReadKey();

        }

        private static void CalculatePsiWithCSharp()
        {
            var cylList = new List<CylinderMeasurement>();
            for (int i = 0; i < _iterations; i++)
            {

                cylList.Add(CreateCylinder(1, 4.005, 4.010, 8.000, 79060, CylinderType.FourFourEightCylinder));
                cylList.Add(CreateCylinder(2, 4.008, 4.006, 8.000, 82630, CylinderType.FourFourEightCylinder));
                cylList.Add(CreateCylinder(3, 5.967, 5.984, 12, 137970, CylinderType.SixSixTwelveCylinder));
                cylList.Add(CreateCylinder(4, 3.636, 3.722, 6.891, 57670, CylinderType.ThreeThreeSixCylinder));
                cylList.Add(CreateCylinder(5, 1.982, 2.022, 2.000, 8540, CylinderType.TwoTwoTwoCylinder));
                cylList.Add(CreateCylinder(6, 6.036, 5.975, 12.000, 100370, CylinderType.SixSixTwelveCylinder));
                cylList.Add(CreateCylinder(7, 3.999, 3.998, 8.000, 71830, CylinderType.FourFourEightCylinder));
                cylList.Add(CreateCylinder(8, 1.994, 1.992, 2.000, 16320, CylinderType.TwoTwoTwoCylinder));
            }


           var sw = new Stopwatch();
            sw.Start();
            CylinderCalculator.UpdateCylinders(cylList);
            sw.Stop();
            Console.WriteLine("Calc time for {0} Iterations in C#:{1}", _iterations, sw.ElapsedMilliseconds);


        }


        private static CylinderMeasurement CreateCylinder(int fakeId, double wA, double wB, double h, int load,
            CylinderType cylType)
        {
            var cyl = new CylinderMeasurement(wA, wB, h);
            cyl.UpdateLoadPoundsAndTypeEnum(load, cylType);
            //fake the id, need it for the assert comparison
            cyl.GetType().GetProperty("Id").SetValue(cyl, fakeId);
            return cyl;
        }



        private static void CalculatePsiWithFSharp()
        {
            var cylList = new List<CylinderMeasurement>();
            for (int i = 0; i < _iterations; i++)
            {
                cylList.Add(CreateCylinder(1, 4.005, 4.010, 8.000, 79060, CylinderType.FourFourEightCylinder));
                cylList.Add(CreateCylinder(2, 4.008, 4.006, 8.000, 82630, CylinderType.FourFourEightCylinder));
                cylList.Add(CreateCylinder(3, 5.967, 5.984, 12, 137970, CylinderType.SixSixTwelveCylinder));
                cylList.Add(CreateCylinder(4, 3.636, 3.722, 6.891, 57670, CylinderType.ThreeThreeSixCylinder));
                cylList.Add(CreateCylinder(5, 1.982, 2.022, 2.000, 8540, CylinderType.TwoTwoTwoCylinder));
                cylList.Add(CreateCylinder(6, 6.036, 5.975, 12.000, 100370, CylinderType.SixSixTwelveCylinder));
                cylList.Add(CreateCylinder(7, 3.999, 3.998, 8.000, 71830, CylinderType.FourFourEightCylinder));
                cylList.Add(CreateCylinder(8, 1.994, 1.992, 2.000, 16320, CylinderType.TwoTwoTwoCylinder));

            }

            var sw = new Stopwatch();
            sw.Start();
            Cylinders.calcPsi.getPsi(cylList.ToArray());
            sw.Stop();
            Console.WriteLine("Calc time for {0} Iterations in F#:{1}", _iterations,sw.ElapsedMilliseconds);

        }
    }
}
