﻿namespace Cylinders

open System
open System.IO
open AreaCalculationDomain


module calcPsi = 
    // Potential formualas
    let fourFourEightFormula WA WB = 3.14159*((WA+WB)/2.)/2.*((WA+WB)/2./2.)               
    let sixSixTwelveFormula WA WB = 3.14159*((WA+WB)/2.)/2.*((WA+WB)/2./2.)               
    let threeThreeSixFormula (WA:float) (WB:float) = WA*WB
    let twoTwoTwoFormula WA WB = ((WA+WB)/2.)*((WA+WB)/2.)

    // ratio
    let ratioFormula height widthA widthB = if (height > 0. && (widthA + widthB > 0.)) then 
                                                Some(Math.Round(height / ((widthA + widthB)/2.), 2))
                                            else 
                                                None

    // tolerance
    let tolerance (ratioValue:float option) = match ratioValue with 
                                                    | _ when (ratioValue.IsSome && ratioValue.Value > 1.94) -> 1.
                                                    | _ when (ratioValue.IsSome && ratioValue.Value < 1.) -> 1.
                                                    | _ -> 0.979

    // Function to evaluate an individual cylinder
    // Calculate the base area, the ratio, the tolerance factor, and finally the PSI. 
    // Update the PSI, and return the original cylinder information.
    let calculatePsi (cylinder:CylinderMeasurement) = 
        let formula = match cylinder.CylinderType with 
                            | CylinderType.FourFourEightCylinder -> fourFourEightFormula 
                            | CylinderType.SixSixTwelveCylinder -> sixSixTwelveFormula 
                            | CylinderType.ThreeThreeSixCylinder -> threeThreeSixFormula 
                            | CylinderType.TwoTwoTwoCylinder -> twoTwoTwoFormula 
                            | _ -> failwith "Unknown cylinder"

        let basearea = formula cylinder.WidthA cylinder.WidthB
        let ratio = ratioFormula cylinder.Height cylinder.WidthA cylinder.WidthB
        let toleranceFactor = tolerance ratio

        let PSI = Math.Round((float)cylinder.LoadPounds/basearea/1000. * toleranceFactor, 2)*1000.

        cylinder.Psi <- PSI
        cylinder

    // Map evaluate to handle all given cylinders. 
    let getPsi (cylinders:CylinderMeasurement[]) = Array.Parallel.map calculatePsi cylinders
